@@ -73,7 +73,7 @@ int loglevel;
  * 	@param msg The error message to print (printf style)
  * 	@param ... Any args to msg (printf style)
  */
-void usage(FILE* fd, const char* msg, ...)
+static void usage(FILE* fd, const char* msg, ...)
 {
 	/* List of args to msg */
 	va_list ap;
@@ -131,17 +131,9 @@ void flog(int level, const char *format, ...)
 	}
 
 	if (logfacility) {
-		sigset_t sigset;
-
-		sigemptyset(&sigset);
-		sigaddset(&sigset, SIGUSR2);
-		sigprocmask(SIG_BLOCK, &sigset, NULL);
-
 		va_start(ap, format);
 		vsyslog(LOG_MAKEPRI(logfacility, level), format, ap);
 		va_end(ap);
-
-		sigprocmask(SIG_UNBLOCK, &sigset, NULL);
 	}
 }
 
@@ -314,27 +306,6 @@ static int cgre_is_unchanged_child(pid_t pid)
 	return 0;
 }
 
-static int cgre_change_cgroup(const uid_t uid, const gid_t gid, char *procname,
-					const pid_t pid)
-{
-	int ret;
-	sigset_t sigset;
-
-	/*
-	 * For avoiding the deadlock, protect cdgroup_change_cgroup_
-	 * ~uid_gid_flags() by blocking SIGUSR2 signal.
-	 */
-	sigemptyset(&sigset);
-	sigaddset(&sigset, SIGUSR2);
-	sigprocmask(SIG_BLOCK, &sigset, NULL);
-
-	ret = cgroup_change_cgroup_flags(uid, gid, procname, pid,
-						 CGFLAG_USECACHE);
-	sigprocmask(SIG_UNBLOCK, &sigset, NULL);
-
-	return ret;
-}
-
 /**
  * Process an event from the kernel, and determine the correct UID/GID/PID to
  * pass to libcgroup.  Then, libcgroup will decide the cgroup to move the PID
@@ -431,19 +402,20 @@ int cgre_process_event(const struct proc_event *ev, const int type)
 	default:
 		break;
 	}
-	ret = cgre_change_cgroup(euid, egid, procname, pid);
-	if (ret) {
-		/*
-		 * TODO: add some supression, do not spam log when every group
-		 * change fails
-		 */
-		flog(LOG_WARNING, "Cgroup change for PID: %d, UID: %d, GID: %d"
-			" FAILED! (Error Code: %d)", log_pid, log_uid, log_gid,
-			ret);
+	ret = cgroup_change_cgroup_flags(euid, egid, procname, pid,
+						 CGFLAG_USECACHE);
+	if ((ret == ECGOTHER) && (errno == ESRCH)) {
+		/* A process finished already and that is not a problem. */
+		ret = 0;
+	} else if (ret) {
+		flog(LOG_WARNING, "Cgroup change for PID: %d, UID: %d, GID: %d,"
+			" PROCNAME: %s FAILED! (Error Code: %d)",
+			log_pid, log_uid, log_gid, procname, ret);
 	} else {
+		flog(LOG_INFO, "Cgroup change for PID: %d, UID: %d, GID: %d,"
+			" PROCNAME: %s OK",
+			log_pid, log_uid, log_gid, procname);
 		ret = cgre_store_parent_info(pid);
-		flog(LOG_INFO, "Cgroup change for PID: %d, UID: %d, GID: %d OK",
-			log_pid, log_uid, log_gid);
 	}
 	free(procname);
 	return ret;
@@ -456,7 +428,7 @@ int cgre_process_event(const struct proc_event *ev, const int type)
  * 	@param cn_hdr The netlink message
  * 	@return 0 on success, > 0 on error
  */
-int cgre_handle_msg(struct cn_msg *cn_hdr)
+static int cgre_handle_msg(struct cn_msg *cn_hdr)
 {
 	/* The event to consider */
 	struct proc_event *ev;
@@ -502,7 +474,7 @@ int cgre_handle_msg(struct cn_msg *cn_hdr)
 	return ret;
 }
 
-int cgre_receive_netlink_msg(int sk_nl)
+static int cgre_receive_netlink_msg(int sk_nl)
 {
 	char buff[BUFF_SIZE];
 	size_t recv_len;
@@ -548,7 +520,7 @@ int cgre_receive_netlink_msg(int sk_nl)
 	return 0;
 }
 
-void cgre_receive_unix_domain_msg(int sk_unix)
+static void cgre_receive_unix_domain_msg(int sk_unix)
 {
 	int flags;
 	int fd_client;
@@ -561,11 +533,11 @@ void cgre_receive_unix_domain_msg(int sk_unix)
 	caddr_len = sizeof(caddr);
 	fd_client = accept(sk_unix, (struct sockaddr *)&caddr, &caddr_len);
 	if (fd_client < 0) {
-		cgroup_dbg("accept error");
+		cgroup_dbg("accept error: %s\n", strerror(errno));
 		return;
 	}
 	if (read(fd_client, &pid, sizeof(pid)) < 0) {
-		cgroup_dbg("read error");
+		cgroup_dbg("read error: %s\n", strerror(errno));
 		goto close;
 	}
 	sprintf(path, "/proc/%d", pid);
@@ -574,7 +546,7 @@ void cgre_receive_unix_domain_msg(int sk_unix)
 		goto close;
 	}
 	if (read(fd_client, &flags, sizeof(flags)) < 0) {
-		cgroup_dbg("read error");
+		cgroup_dbg("read error: %s\n", strerror(errno));
 		goto close;
 	}
 	if (cgre_store_unchanged_process(pid, flags))
@@ -582,7 +554,7 @@ void cgre_receive_unix_domain_msg(int sk_unix)
 
 	if (write(fd_client, CGRULE_SUCCESS_STORE_PID,
 			sizeof(CGRULE_SUCCESS_STORE_PID)) < 0) {
-		cgroup_dbg("write error");
+		cgroup_dbg("write error: %s\n", strerror(errno));
 		goto close;
 	}
 close:
@@ -590,7 +562,7 @@ close:
 	return;
 }
 
-int cgre_create_netlink_socket_process_msg()
+static int cgre_create_netlink_socket_process_msg(void)
 {
 	int sk_nl = 0, sk_unix = 0, sk_max;
 	struct sockaddr_nl my_nla;
@@ -601,6 +573,7 @@ int cgre_create_netlink_socket_process_msg()
 	enum proc_cn_mcast_op *mcop_msg;
 	struct sockaddr_un saddr;
 	fd_set fds, readfds;
+	sigset_t sigset;
 
 	/*
 	 * Create an endpoint for communication. Use the kernel user
@@ -610,7 +583,7 @@ int cgre_create_netlink_socket_process_msg()
 	 */
 	sk_nl = socket(PF_NETLINK, SOCK_DGRAM, NETLINK_CONNECTOR);
 	if (sk_nl == -1) {
-		cgroup_dbg("socket sk_nl error");
+		cgroup_dbg("socket sk_nl error: %s\n", strerror(errno));
 		return rc;
 	}
 
@@ -620,7 +593,7 @@ int cgre_create_netlink_socket_process_msg()
 	my_nla.nl_pad = 0;
 
 	if (bind(sk_nl, (struct sockaddr *)&my_nla, sizeof(my_nla)) < 0) {
-		cgroup_dbg("binding sk_nl error");
+		cgroup_dbg("binding sk_nl error: %s\n", strerror(errno));
 		goto close_and_exit;
 	}
 
@@ -647,7 +620,8 @@ int cgre_create_netlink_socket_process_msg()
 	cgroup_dbg("sending netlink message len=%d, cn_msg len=%d\n",
 		nl_hdr->nlmsg_len, (int) sizeof(struct cn_msg));
 	if (send(sk_nl, nl_hdr, nl_hdr->nlmsg_len, 0) != nl_hdr->nlmsg_len) {
-		cgroup_dbg("failed to send proc connector mcast ctl op!\n");
+		cgroup_dbg("failed to send proc connector mcast ctl op!: %s\n",
+			strerror(errno));
 		goto close_and_exit;
 	}
 	cgroup_dbg("sent\n");
@@ -657,7 +631,7 @@ int cgre_create_netlink_socket_process_msg()
 	 */
 	sk_unix = socket(PF_UNIX, SOCK_STREAM, 0);
 	if (sk_unix < 0) {
-		cgroup_dbg("socket sk_unix error");
+		cgroup_dbg("socket sk_unix error: %s\n", strerror(errno));
 		goto close_and_exit;
 	}
 	memset(&saddr, 0, sizeof(saddr));
@@ -666,11 +640,11 @@ int cgre_create_netlink_socket_process_msg()
 	unlink(CGRULE_CGRED_SOCKET_PATH);
 	if (bind(sk_unix, (struct sockaddr *)&saddr,
 	    sizeof(saddr.sun_family) + strlen(CGRULE_CGRED_SOCKET_PATH)) < 0) {
-		cgroup_dbg("binding sk_unix error");
+		cgroup_dbg("binding sk_unix error: %s\n", strerror(errno));
 		goto close_and_exit;
 	}
 	if (listen(sk_unix, 1) < 0) {
-		cgroup_dbg("listening sk_unix error");
+		cgroup_dbg("listening sk_unix error: %s\n", strerror(errno));
 		goto close_and_exit;
 	}
 	FD_ZERO(&readfds);
@@ -680,10 +654,20 @@ int cgre_create_netlink_socket_process_msg()
 		sk_max = sk_unix;
 	else
 		sk_max = sk_nl;
+
+	sigemptyset(&sigset);
+	sigaddset(&sigset, SIGUSR2);
 	for(;;) {
+		/*
+		 * For avoiding the deadlock and "Interrupted system call"
+		 * error, restrict the effective range of SIGUSR2 signal.
+		 */
+		sigprocmask(SIG_UNBLOCK, &sigset, NULL);
+		sigprocmask(SIG_BLOCK, &sigset, NULL);
+
 		memcpy(&fds, &readfds, sizeof(fd_set));
 		if (select(sk_max + 1, &fds, NULL, NULL, NULL) < 0) {
-			cgroup_dbg("selecting error");
+			cgroup_dbg("selecting error: %s\n", strerror(errno));
 			goto close_and_exit;
 		}
 		if (FD_ISSET(sk_nl, &fds)) {
@@ -1050,9 +1034,8 @@ int main(int argc, char *argv[])
 	 * Set up the signal handler to reload the cached rules upon reception
 	 * of a SIGUSR2 signal.
 	 */
+	memset(&sa, 0, sizeof(sa));
 	sa.sa_handler = &cgre_flash_rules;
-	sa.sa_flags = 0;
-	sa.sa_restorer = NULL;
 	sigemptyset(&sa.sa_mask);
 	if ((ret = sigaction(SIGUSR2, &sa, NULL))) {
 		flog(LOG_ERR, "Failed to set up signal handler for SIGUSR2."
