@@ -1,3 +1,18 @@
+/*
+ * Copyright Red Hat, Inc. 2009
+ *
+ * Authors:	Ivana Hutarova Varekova <varekova@redhat.com>
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of version 2.1 of the GNU Lesser General Public License
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it would be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *
+ */
+
 #include <libcgroup.h>
 #include <libcgroup-internal.h>
 
@@ -7,6 +22,8 @@
 #include <pwd.h>
 #include <sys/types.h>
 #include <errno.h>
+#include <unistd.h>
+#include <grp.h>
 
 #include "tools-common.h"
 
@@ -14,7 +31,7 @@ int main(int argc, char *argv[])
 {
 	int ret = 0;
 	int i, j;
-	char c;
+	int c;
 
 	/* Structure to get GID from group name */
 	struct group *grp = NULL;
@@ -27,10 +44,13 @@ int main(int argc, char *argv[])
 	uid_t tuid = CGRULE_INVALID, auid = CGRULE_INVALID;
 	gid_t tgid = CGRULE_INVALID, agid = CGRULE_INVALID;
 
-	struct cgroup_group_spec *cgroup_list[CG_HIER_MAX];
+	struct cgroup_group_spec **cgroup_list;
 	struct cgroup *cgroup;
 	struct cgroup_controller *cgc;
 
+	/* approximation of max. numbers of groups that will be created */
+	int capacity = argc;
+
 	/* no parametr on input */
 	if (argc < 2) {
 		fprintf(stderr, "Usage is %s "
@@ -39,8 +59,12 @@ int main(int argc, char *argv[])
 			argv[0]);
 		return -1;
 	}
+	cgroup_list = calloc(capacity, sizeof(struct cgroup_group_spec *));
+	if (cgroup_list == NULL) {
+		fprintf(stderr, "%s: out of memory\n", argv[0]);
+		return -1;
+	}
 
-	memset(cgroup_list, 0, sizeof(cgroup_list));
 	/* parse arguments */
 	while ((c = getopt(argc, argv, "a:t:g:")) > 0) {
 		switch (c) {
@@ -112,7 +136,8 @@ int main(int argc, char *argv[])
 			}
 			break;
 		case 'g':
-			if (parse_cgroup_spec(cgroup_list, optarg)) {
+			ret = parse_cgroup_spec(cgroup_list, optarg, capacity);
+			if (ret) {
 				fprintf(stderr, "%s: "
 					"cgroup controller and path"
 					"parsing failed (%s)\n",
@@ -148,7 +173,7 @@ int main(int argc, char *argv[])
 	}
 
 	/* for each new cgroup */
-	for (i = 0; i < CG_HIER_MAX; i++) {
+	for (i = 0; i < capacity; i++) {
 		if (!cgroup_list[i])
 			break;
 
@@ -195,9 +220,12 @@ int main(int argc, char *argv[])
 		cgroup_free(&cgroup);
 	}
 err:
-	for (i = 0; i < CG_HIER_MAX; i++) {
-		if (cgroup_list[i])
-			cgroup_free_group_spec(cgroup_list[i]);
+	if (cgroup_list) {
+		for (i = 0; i < capacity; i++) {
+			if (cgroup_list[i])
+				cgroup_free_group_spec(cgroup_list[i]);
+		}
+		free(cgroup_list);
 	}
 	return ret;
 }
